"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const magic_string_1 = __importDefault(require("magic-string"));
const pluginutils_1 = require("@rollup/pluginutils");
function escape(str) {
    return str.replace(/[-[\]/{}()*+?.\\^$|]/g, '\\$&');
}
function ensureFunction(functionOrValue) {
    if (typeof functionOrValue === 'function')
        return functionOrValue;
    return () => functionOrValue;
}
function longest(a, b) {
    return b.length - a.length;
}
function getReplacements(options) {
    if (options.values) {
        return Object.assign({}, options.values);
    }
    const values = Object.assign({}, options);
    delete values.delimiters;
    delete values.preventAssignment;
    delete values.include;
    delete values.exclude;
    delete values.sourcemap;
    delete values.sourceMap;
    return values;
}
function mapToFunctions(object) {
    return Object.keys(object).reduce((fns, key) => {
        const functions = Object.assign({}, fns);
        functions[key] = ensureFunction(object[key]);
        return functions;
    }, {});
}
function replace(options = {}) {
    // eslint-disable-next-line no-param-reassign
    options = Object.assign({ include: ['./src/**/*'], preventAssignment: true }, options);
    const filter = pluginutils_1.createFilter(options.include, options.exclude);
    const { delimiters, preventAssignment } = options;
    const replaced = getReplacements(options);
    // add env variables in options
    const vars = Object.keys(replaced).reduce((a, c) => {
        // eslint-disable-next-line no-param-reassign
        a[`process.env.${c}`] = JSON.stringify(replaced[c]);
        return a;
    }, {});
    // add NUXT_ENV_ prefix env variables
    Object.keys(process.env).forEach((key) => {
        if (key.startsWith('NUXT_ENV')) {
            vars[`process.env.${key}`] = JSON.stringify(process.env[key]);
        }
    });
    // add empty process.env
    if (typeof vars['process.env'] === 'undefined') {
        vars['process.env'] = JSON.stringify({});
    }
    const functionValues = mapToFunctions(vars);
    const keys = Object.keys(functionValues).sort(longest).map(escape);
    const lookahead = preventAssignment ? '(?!\\s*=[^=])' : '';
    const pattern = delimiters
        ? new RegExp(`${escape(delimiters[0])}(${keys.join('|')})${escape(delimiters[1])}${lookahead}`, 'g')
        : new RegExp(`\\b(${keys.join('|')})\\b${lookahead}`, 'g');
    function codeHasReplacements(code, id, magicString) {
        let result = false;
        let match;
        // eslint-disable-next-line no-cond-assign
        while ((match = pattern.exec(code))) {
            result = true;
            const start = match.index;
            const end = start + match[0].length;
            const replacement = String(functionValues[match[1]](id));
            magicString.overwrite(start, end, replacement);
        }
        return result;
    }
    function isSourceMapEnabled() {
        return options.sourceMap !== false && options.sourcemap !== false;
    }
    function executeReplacement(code, id) {
        const magicString = new magic_string_1.default(code);
        if (!codeHasReplacements(code, id, magicString)) {
            return null;
        }
        const result = { code: magicString.toString() };
        if (isSourceMapEnabled()) {
            result.map = magicString.generateMap({ hires: true });
        }
        return result;
    }
    return {
        name: 'vite-plugin-nuxt-env',
        transform(code, id) {
            if (!keys.length)
                return null;
            if (!filter(id))
                return null;
            return executeReplacement(code, id);
        },
    };
}
exports.default = replace;
